package com.ips.system.service.impl;

import java.util.*;
import java.util.concurrent.CompletableFuture;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ips.common.utils.DateUtils;
import com.ips.common.utils.StringUtils;
import com.ips.system.domain.AlgorithmConfig;
import com.ips.system.dto.AlgorithmParamsDto;
import com.ips.system.dto.ParentFolderInfoDTO;
import com.ips.system.service.IAlgorithmConfigService;
import com.ips.system.utils.AlgorithmCaller;
import com.ips.system.utils.CommonUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ips.system.mapper.ExtractedFeaturesMapper;
import com.ips.system.domain.ExtractedFeatures;
import com.ips.system.service.IExtractedFeaturesService;

/**
 * 特征提取Service业务层处理
 * 
 * @author Allen
 * @date 2025-05-21
 */
@Service
public class ExtractedFeaturesServiceImpl implements IExtractedFeaturesService 
{
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private ExtractedFeaturesMapper extractedFeaturesMapper;

    @Autowired
    private IAlgorithmConfigService algorithmConfigService;

    /**
     * 查询特征提取
     * 
     * @param id 特征提取主键
     * @return 特征提取
     */
    @Override
    public ExtractedFeatures selectExtractedFeaturesById(Long id)
    {
        return extractedFeaturesMapper.selectExtractedFeaturesById(id);
    }

    /**
     * 查询特征提取列表
     * 
     * @param extractedFeatures 特征提取
     * @return 特征提取
     */
    @Override
    public List<ExtractedFeatures> selectExtractedFeaturesList(ExtractedFeatures extractedFeatures)
    {
        return extractedFeaturesMapper.selectExtractedFeaturesList(extractedFeatures);
    }

    /**
     * 新增特征提取
     * 
     * @param extractedFeatures 特征提取
     * @return 结果
     */
    @Override
    public int insertExtractedFeatures(ExtractedFeatures extractedFeatures)
    {
        extractedFeatures.setCreateTime(DateUtils.getNowDate());
        return extractedFeaturesMapper.insertExtractedFeatures(extractedFeatures);
    }

    /**
     * 修改特征提取
     * 
     * @param extractedFeatures 特征提取
     * @return 结果
     */
    @Override
    public int updateExtractedFeatures(ExtractedFeatures extractedFeatures)
    {
        extractedFeatures.setUpdateTime(DateUtils.getNowDate());
        return extractedFeaturesMapper.updateExtractedFeatures(extractedFeatures);
    }

    /**
     * 批量删除特征提取
     * 
     * @param ids 需要删除的特征提取主键
     * @return 结果
     */
    @Override
    public int deleteExtractedFeaturesByIds(Long[] ids)
    {
        return extractedFeaturesMapper.deleteExtractedFeaturesByIds(ids);
    }

    /**
     * 删除特征提取信息
     * 
     * @param id 特征提取主键
     * @return 结果
     */
    @Override
    public int deleteExtractedFeaturesById(Long id)
    {
        return extractedFeaturesMapper.deleteExtractedFeaturesById(id);
    }

    @Override
    public String run(Long id) throws JsonProcessingException {
        ExtractedFeatures extractedFeatures = extractedFeaturesMapper.selectExtractedFeaturesById(id);
        if(extractedFeatures == null || extractedFeatures.getAlgorithmId() == null){
            return "无法找到该任务,id:"+id;
        }
        Long algorithmId = extractedFeatures.getAlgorithmId();
        AlgorithmConfig algorithmConfig = algorithmConfigService.selectAlgorithmConfigById(algorithmId);
        extractedFeatures.setStartTime(new Date());
        extractedFeatures.setStatus("1");
        this.updateExtractedFeatures(extractedFeatures);

        CompletableFuture.runAsync(() -> {
            // 异步逻辑
            doRun(algorithmConfig, extractedFeatures);
        });
        return null;
    }

    private void doRun(AlgorithmConfig algorithmConfig, ExtractedFeatures extractedFeatures) {
        String algorithmPath = algorithmConfig.getAlgorithmPath();

        // 组装json
        String inputPath = extractedFeatures.getInputPath();
        String outputPath = extractedFeatures.getOutputPath();
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> params = new HashMap<>(0);

        String errorMsg = "";
        try {
            if (StringUtils.isNotEmpty(extractedFeatures.getAlgorithmParams())) {

                params = objectMapper.readValue(
                        extractedFeatures.getAlgorithmParams(),
                        new TypeReference<Map<String, Object>>() {
                        }
                );
            }
            AlgorithmParamsDto algorithmParamsDto = new AlgorithmParamsDto(algorithmConfig.getAlgorithmName(), inputPath, outputPath, params);
            // 对象 → JSON 字符串
            String json = objectMapper.writeValueAsString(algorithmParamsDto);
            // 处理算法
            errorMsg = AlgorithmCaller.executeAlgorithm(algorithmConfig.getAlgorithmName(),algorithmPath, json);
        } catch (JsonProcessingException e) {
            logger.error("格式化失败", e);
            errorMsg = "格式化失败";
        }
        //处理结果
        extractedFeatures.setRemark(StringUtils.substring(errorMsg,0,100));
        if (StringUtils.isEmpty(errorMsg)) {
            extractedFeatures.setStatus("2");
        } else {
            logger.error("算法调用失败，错误原因：{}", errorMsg);
            extractedFeatures.setStatus("3");
        }
        extractedFeatures.setEndTime(new Date());
        this.updateExtractedFeatures(extractedFeatures);
    }


    @Override
    public ExtractedFeatures getResultDetails(Long id) {
        ExtractedFeatures extractedFeatures = this.selectExtractedFeaturesById(id);
        String inputPath = extractedFeatures.getInputPath();
        List<ParentFolderInfoDTO> parentFolderInfoDTOList = CommonUtils.getFolderStructureWithFilter(inputPath, Arrays.asList("dat"));
        extractedFeatures.setParentFolderInfoDTOList(parentFolderInfoDTOList);
        return extractedFeatures;
    }

    @Override
    public Boolean getTaskStatusIsChanged(Long id) {
        return extractedFeaturesMapper.getTaskStatusIsChanged(id);
    }
}
