package com.ips.system.service.impl;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.CompletableFuture;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ips.common.utils.DateUtils;
import com.ips.common.utils.StringUtils;
import com.ips.system.domain.AlgorithmConfig;
import com.ips.system.domain.ExtractedFeatures;
import com.ips.system.dto.AlgorithmParamsDto;
import com.ips.system.dto.FileInfoDTO;
import com.ips.system.dto.FolderInfoDTO;
import com.ips.system.service.IAlgorithmConfigService;
import com.ips.system.utils.AlgorithmCaller;
import com.ips.system.utils.CommonUtils;
import org.apache.poi.ss.formula.functions.T;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import com.ips.system.mapper.PreprocessedMapper;
import com.ips.system.domain.Preprocessed;
import com.ips.system.service.IPreprocessedService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 数据预处理Service业务层处理
 * 
 * @author Allen
 * @date 2025-05-21
 */
@Service
public class PreprocessedServiceImpl implements IPreprocessedService 
{

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private PreprocessedMapper preprocessedMapper;

    @Autowired
    private IAlgorithmConfigService algorithmConfigService;

    /**
     * 查询数据预处理
     * 
     * @param id 数据预处理主键
     * @return 数据预处理
     */
    @Override
    public Preprocessed selectPreprocessedById(Long id)
    {
        return preprocessedMapper.selectPreprocessedById(id);
    }

    /**
     * 查询数据预处理列表
     * 
     * @param preprocessed 数据预处理
     * @return 数据预处理
     */
    @Override
    public List<Preprocessed> selectPreprocessedList(Preprocessed preprocessed)
    {
        return preprocessedMapper.selectPreprocessedList(preprocessed);
    }

    /**
     * 新增数据预处理
     * 
     * @param preprocessed 数据预处理
     * @return 结果
     */
    @Override
    public int insertPreprocessed(Preprocessed preprocessed)
    {
        preprocessed.setCreateTime(DateUtils.getNowDate());
        preprocessed.setStatus("0");
        return preprocessedMapper.insertPreprocessed(preprocessed);
    }

    /**
     * 修改数据预处理
     * 
     * @param preprocessed 数据预处理
     * @return 结果
     */
    @Override
    public int updatePreprocessed(Preprocessed preprocessed)
    {
        preprocessed.setUpdateTime(DateUtils.getNowDate());
        return preprocessedMapper.updatePreprocessed(preprocessed);
    }

    /**
     * 批量删除数据预处理
     * 
     * @param ids 需要删除的数据预处理主键
     * @return 结果
     */
    @Override
    public int deletePreprocessedByIds(Long[] ids)
    {
        return preprocessedMapper.deletePreprocessedByIds(ids);
    }

    /**
     * 删除数据预处理信息
     * 
     * @param id 数据预处理主键
     * @return 结果
     */
    @Override
    public int deletePreprocessedById(Long id)
    {
        return preprocessedMapper.deletePreprocessedById(id);
    }

    @Override
    public String run(Long id) {
        Preprocessed preprocessed = preprocessedMapper.selectPreprocessedById(id);
        if(preprocessed == null || preprocessed.getAlgorithmId() == null){
            return "无法找到该任务,id:"+id;
        }
        Long algorithmId = preprocessed.getAlgorithmId();
        AlgorithmConfig algorithmConfig = algorithmConfigService.selectAlgorithmConfigById(algorithmId);
        preprocessed.setStartTime(new Date());
        preprocessed.setStatus("1");
        this.updatePreprocessed(preprocessed);

        CompletableFuture.runAsync(() -> {
            // 异步逻辑
            doRun(algorithmConfig, preprocessed);
        });
        return null;
    }

    private void doRun(AlgorithmConfig algorithmConfig, Preprocessed preprocessed) {
        String algorithmPath = algorithmConfig.getAlgorithmPath();

        // 组装json
        String inputPath = preprocessed.getInputPath();
        String outputPath = preprocessed.getOutputPath();
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> params = new HashMap<>(0);

        String errorMsg = "";
        try {
            if (StringUtils.isNotEmpty(preprocessed.getAlgorithmParams())) {

                params = objectMapper.readValue(
                        preprocessed.getAlgorithmParams(),
                        new TypeReference<Map<String, Object>>() {
                        }
                );
            }
            AlgorithmParamsDto algorithmParamsDto = new AlgorithmParamsDto(algorithmConfig.getAlgorithmName(), inputPath, outputPath, params);
            // 对象 → JSON 字符串
            String json = objectMapper.writeValueAsString(algorithmParamsDto);
            // 处理算法
            errorMsg = AlgorithmCaller.executeAlgorithm(algorithmConfig.getAlgorithmName(),algorithmPath, json);
        } catch (JsonProcessingException e) {
            logger.error("格式化失败", e);
            errorMsg = "格式化失败";
        }
        //处理结果
        preprocessed.setRemark(StringUtils.substring(errorMsg,0,100));
        if (StringUtils.isEmpty(errorMsg)) {
            preprocessed.setStatus("2");
        } else {
            logger.error("算法调用失败，错误原因：{}", errorMsg);
            preprocessed.setStatus("3");
        }
        preprocessed.setEndTime(new Date());
        this.updatePreprocessed(preprocessed);
    }

    @Override
    public Preprocessed getResultDetails(Long id) {
        Preprocessed preprocessed = this.selectPreprocessedById(id);
        String inputPath = preprocessed.getInputPath();
        try {
            List<FileInfoDTO> fileInfoDTOList = CommonUtils.getSortedFiles(inputPath, "dat");
            preprocessed.setFolderInfoDTO(fileInfoDTOList);
        } catch (IOException e) {
            logger.error("读取文件夹错误", e);
            return null;
        }
        return preprocessed;
    }

    @Override
    public Boolean getTaskStatusIsChanged(Long id) {
        return preprocessedMapper.getTaskStatusIsChanged(id);
    }
}
