package com.ips.system.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ips.common.utils.DateUtils;
import com.ips.common.utils.StringUtils;
import com.ips.system.domain.AlgorithmConfig;
import com.ips.system.dto.AlgorithmParamsDto;
import com.ips.system.service.IAlgorithmConfigService;
import com.ips.system.utils.AlgorithmCaller;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ips.system.mapper.TimeFrequencyImagesMapper;
import com.ips.system.domain.TimeFrequencyImages;
import com.ips.system.service.ITimeFrequencyImagesService;

/**
 * 时频图生成Service业务层处理
 * 
 * @author Allen
 * @date 2025-05-28
 */
@Service
public class TimeFrequencyImagesServiceImpl implements ITimeFrequencyImagesService 
{
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private TimeFrequencyImagesMapper timeFrequencyImagesMapper;

    @Autowired
    private IAlgorithmConfigService algorithmConfigService;


    /**
     * 查询时频图生成
     * 
     * @param id 时频图生成主键
     * @return 时频图生成
     */
    @Override
    public TimeFrequencyImages selectTimeFrequencyImagesById(Long id)
    {
        return timeFrequencyImagesMapper.selectTimeFrequencyImagesById(id);
    }

    /**
     * 查询时频图生成列表
     * 
     * @param timeFrequencyImages 时频图生成
     * @return 时频图生成
     */
    @Override
    public List<TimeFrequencyImages> selectTimeFrequencyImagesList(TimeFrequencyImages timeFrequencyImages)
    {
        return timeFrequencyImagesMapper.selectTimeFrequencyImagesList(timeFrequencyImages);
    }

    /**
     * 新增时频图生成
     * 
     * @param timeFrequencyImages 时频图生成
     * @return 结果
     */
    @Override
    public int insertTimeFrequencyImages(TimeFrequencyImages timeFrequencyImages)
    {
        timeFrequencyImages.setCreateTime(DateUtils.getNowDate());
        return timeFrequencyImagesMapper.insertTimeFrequencyImages(timeFrequencyImages);
    }

    /**
     * 修改时频图生成
     * 
     * @param timeFrequencyImages 时频图生成
     * @return 结果
     */
    @Override
    public int updateTimeFrequencyImages(TimeFrequencyImages timeFrequencyImages)
    {
        timeFrequencyImages.setUpdateTime(DateUtils.getNowDate());
        return timeFrequencyImagesMapper.updateTimeFrequencyImages(timeFrequencyImages);
    }

    /**
     * 批量删除时频图生成
     * 
     * @param ids 需要删除的时频图生成主键
     * @return 结果
     */
    @Override
    public int deleteTimeFrequencyImagesByIds(Long[] ids)
    {
        return timeFrequencyImagesMapper.deleteTimeFrequencyImagesByIds(ids);
    }

    /**
     * 删除时频图生成信息
     * 
     * @param id 时频图生成主键
     * @return 结果
     */
    @Override
    public int deleteTimeFrequencyImagesById(Long id)
    {
        return timeFrequencyImagesMapper.deleteTimeFrequencyImagesById(id);
    }

    @Override
    public String run(Long id)  {
        TimeFrequencyImages timeFrequencyImages = timeFrequencyImagesMapper.selectTimeFrequencyImagesById(id);
        if(timeFrequencyImages == null || timeFrequencyImages.getAlgorithmId() == null){
            return "无法找到该任务,id:"+id;
        }
        Long algorithmId = timeFrequencyImages.getAlgorithmId();
        AlgorithmConfig algorithmConfig = algorithmConfigService.selectAlgorithmConfigById(algorithmId);
        timeFrequencyImages.setStartTime(new Date());
        timeFrequencyImages.setStatus("1");
        this.updateTimeFrequencyImages(timeFrequencyImages);

        CompletableFuture.runAsync(() -> {
            // 异步逻辑
            doRun(algorithmConfig, timeFrequencyImages);
        });
        return null;
    }

    private void doRun(AlgorithmConfig algorithmConfig, TimeFrequencyImages timeFrequencyImages) {
        String algorithmPath = algorithmConfig.getAlgorithmPath();

        // 组装json
        String inputPath = timeFrequencyImages.getInputPath();
        String outputPath = timeFrequencyImages.getOutputPath();
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> params = new HashMap<>(0);

        String errorMsg = "";
        try {
            if (StringUtils.isNotEmpty(timeFrequencyImages.getAlgorithmParams())) {

                params = objectMapper.readValue(
                        timeFrequencyImages.getAlgorithmParams(),
                        new TypeReference<Map<String, Object>>() {
                        }
                );
            }
            AlgorithmParamsDto algorithmParamsDto = new AlgorithmParamsDto(algorithmConfig.getAlgorithmName(), inputPath, outputPath, params);
            // 对象 → JSON 字符串
            String json = objectMapper.writeValueAsString(algorithmParamsDto);
            // 处理算法
            errorMsg = AlgorithmCaller.executeAlgorithm(algorithmConfig.getAlgorithmName(),algorithmPath, json);
        } catch (JsonProcessingException e) {
            logger.error("格式化失败", e);
            errorMsg = "格式化失败";
        }
        //处理结果
        timeFrequencyImages.setRemark(StringUtils.substring(errorMsg,0,100));
        if (StringUtils.isEmpty(errorMsg)) {
            timeFrequencyImages.setStatus("2");
        } else {
            logger.error("算法调用失败，错误原因：{}", errorMsg);
            timeFrequencyImages.setStatus("3");
        }
        timeFrequencyImages.setEndTime(new Date());
        this.updateTimeFrequencyImages(timeFrequencyImages);
    }

    @Override
    public Boolean getTaskStatusIsChanged(Long id) {
        return timeFrequencyImagesMapper.getTaskStatusIsChanged(id);
    }
}
