package com.ips.system.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Component
public class AlgorithmCaller {
    private static final Logger logger = LoggerFactory.getLogger(AlgorithmCaller.class);
    private static final String DEFAULT_PYTHON = "python";

    /**
     * 获取算法的根目录
     * @param filePath 文件的完整路径，如"d:/xxx/yyy/zzz/dd.py" 或 "d:/xxx/yyy/zzz/program.exe"
     * @return 返回根目录路径，如"d:/xxx/yyy/zzz/"
     */
    public static String getAlgorithmRootPath(String filePath) {
        if (filePath == null || filePath.isEmpty()) {
            throw new IllegalArgumentException("File path cannot be null or empty");
        }

        Path path = Paths.get(filePath);
        Path parentPath = path.getParent();

        if (parentPath != null) {
            return parentPath.toString() + File.separator;
        } else {
            throw new IllegalArgumentException("Invalid file path: " + filePath);
        }
    }


    /**
     * 根据json字符串生成input.json文件到指定目录（解决中文乱码问题）
     *
     * @param jsonName input_xxx.json的xxx部分（算法的名称）
     * @param directory   目标目录
     * @param jsonContent JSON内容字符串
     * @return 生成的JSON文件路径
     */
    public static String generateInputJsonFile(String jsonName, String directory, String jsonContent) throws IOException {
        if (directory == null || directory.isEmpty()) {
            throw new IllegalArgumentException("Directory cannot be null or empty");
        }

        if (jsonContent == null || jsonContent.isEmpty()) {
            throw new IllegalArgumentException("JSON content cannot be null or empty");
        }

        // 确保目录存在
        File dir = new File(directory);
        if (!dir.exists()) {
            boolean created = dir.mkdirs();
            if (!created) {
                throw new IOException("Failed to create directory: " + directory);
            }
        }

        // 创建input.json文件（使用UTF-8编码写入）
        File jsonFile = new File(directory, "input_" + jsonName + ".json");
        try (OutputStreamWriter writer = new OutputStreamWriter(
                Files.newOutputStream(jsonFile.toPath()), StandardCharsets.UTF_8)) {
            writer.write(jsonContent);
        }

        return jsonFile.getAbsolutePath();
    }
    /**
     * 调用Python代码
     *
     * @param pythonScriptPath Python脚本路径
     * @param workingDirectory 工作目录
     * @param jsonContent
     * @return 执行成功返回空字符串，失败返回错误信息
     */
    public static String callPythonScript(String pythonScriptPath, String workingDirectory, String jsonContent) {
        if (pythonScriptPath == null || pythonScriptPath.isEmpty()) {
            throw new IllegalArgumentException("Python script path cannot be null or empty");
        }
//        new ProcessBuilder(exePath, "--config-json", jsonContent);
        ProcessBuilder processBuilder = new ProcessBuilder(DEFAULT_PYTHON, pythonScriptPath, "--config-json", jsonContent);
        processBuilder.directory(new File(workingDirectory));
        processBuilder.redirectErrorStream(true);

        return executeProcess(processBuilder, pythonScriptPath);
    }

    /**
     * 调用EXE程序
     *
     * @param exePath          EXE程序路径
     * @param workingDirectory 工作目录
     * @param jsonContent
     * @return 执行成功返回空字符串，失败返回错误信息
     */
    public static String callExeProgram(String exePath, String workingDirectory, String jsonContent) {
        if (exePath == null || exePath.isEmpty()) {
            throw new IllegalArgumentException("EXE path cannot be null or empty");
        }

        ProcessBuilder processBuilder = new ProcessBuilder(exePath, "--config-json", "\"" + jsonContent.replace("\"", "\\\"") + "\"");
        processBuilder.directory(new File(workingDirectory));
        processBuilder.redirectErrorStream(true);

        return executeProcess(processBuilder, exePath);
    }

    /**
     * 执行进程并处理输出
     * @param processBuilder 进程构建器
     * @param programPath 程序路径（用于日志记录）
     * @return 执行结果
     */
    private static String executeProcess(ProcessBuilder processBuilder, String programPath) {
        try {
            Process process = processBuilder.start();

            // 读取输出
            StringBuilder output = new StringBuilder();
            try (BufferedReader reader = new BufferedReader(
                    new InputStreamReader(process.getInputStream()))) {
                String line;
                while ((line = reader.readLine()) != null) {
                    output.append(line).append("\n");
                }
            }

            // 等待进程结束
            int exitCode = process.waitFor();

            if (exitCode == 0) {
                logger.info("Program executed successfully: {}", programPath);
                return "";
            } else {
                String errorMsg = String.format("Program execution failed with exit code %d. Output: %s",
                        exitCode, output.toString());
                logger.error(errorMsg);
                return errorMsg;
            }
        } catch (IOException | InterruptedException e) {
            String errorMsg = String.format("Failed to execute program: %s. Error: %s",
                    programPath, e.getMessage());
            logger.error(errorMsg, e);
            return errorMsg;
        }
    }

    /**
     * 完整调用流程
     *
     * @param jsonName 算法名称
     * @param programPath 程序路径（可以是.py或.exe）
     * @param jsonContent 输入JSON内容
     * @return 执行结果
     */
    public static String executeAlgorithm(String jsonName,String programPath, String jsonContent) {
        logger.info("调用算法开始，jsonName:{},programPath：{}，jsonContent：{}", jsonName, programPath, jsonContent);
        String errorMsg = "";
        try {
            // 1. 获取工作目录
            String workingDirectory = getAlgorithmRootPath(programPath);

            // 2. 生成输入文件
            generateInputJsonFile(jsonName,workingDirectory, jsonContent);

            // 3. 根据文件类型调用相应程序
            if (programPath.toLowerCase().endsWith(".py")) {
                return callPythonScript(programPath, workingDirectory,jsonContent);
            } else if (programPath.toLowerCase().endsWith(".exe")) {
                return callExeProgram(programPath, workingDirectory,jsonContent);
            } else {
                throw new IllegalArgumentException("Unsupported program type: " + programPath);
            }
        } catch (Exception e) {
            errorMsg = String.format("Algorithm execution failed: %s", e.getMessage());
            logger.error(errorMsg, e);
        }
        return errorMsg;
    }
}