package com.ips.system.utils;

import com.ips.system.dto.FileInfoDTO;
import com.ips.system.dto.FolderInfoDTO;
import com.ips.system.dto.ParentFolderInfoDTO;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CommonUtils {
    public static List<FolderInfoDTO> scanAndSortWithStream(String rootPath, String extension) throws IOException {
        Path rootDir = Paths.get(rootPath);

        return Files.walk(rootDir)
                .filter(p -> Files.isRegularFile(p) && p.toString().endsWith(extension))
                .collect(Collectors.groupingBy(
                        file -> file.getParent().getFileName().toString(),
                        Collectors.mapping(
                                file -> new FileInfoDTO(
                                        file.getFileName().toString(),
                                        file.toString(),
                                        file.toFile().length()
                                ),
                                Collectors.toList()
                        )
                ))
                .entrySet()
                .stream()
                .map(entry -> {
                    FolderInfoDTO folder = new FolderInfoDTO(entry.getKey());
                    // 对文件按fileName排序
                    List<FileInfoDTO> sortedFiles = entry.getValue().stream()
                            .sorted(Comparator.comparing(FileInfoDTO::getFileName))
                            .collect(Collectors.toList());
                    folder.getFiles().addAll(sortedFiles);
                    return folder;
                })
                .sorted(Comparator.comparing(FolderInfoDTO::getFolderName)) // 对文件夹按folderName排序
                .collect(Collectors.toList());
    }
//    public static void main(String[] args) throws IOException {
//        String rootPath = "D:\\ips\\algrothm\\preprocess\\test";
//        String extension = ".dat";
//
//        List<FolderInfoDTO> folders = scanAndSortWithStream(rootPath, extension);
//
//        // 打印结果
//        folders.forEach(folder -> {
//            System.out.println("文件夹: " + folder.getFolderName());
//            folder.getFiles().forEach(file ->
//                    System.out.println("  文件: " + file.getFileName() +
//                            ", 大小: " + file.getFileSize() + " bytes")
//            );
//        });
//    }

    /**
     * 获取文件夹下所有文件（fileName不含后缀），并按文件名（不含后缀）排序
     *
     * @param folderPath 文件夹路径
     * @return 排序后的文件列表
     * @throws IOException 如果读取文件出错
     */
    public static List<FileInfoDTO> getAllSortedFiles(String folderPath) throws IOException {
        // 验证输入参数
        if (folderPath == null) {
            throw new IllegalArgumentException("文件夹路径不能为null");
        }

        Path dir = Paths.get(folderPath);
        if (!Files.isDirectory(dir)) {
            throw new IllegalArgumentException("提供的路径不是一个有效的文件夹: " + folderPath);
        }

        try (Stream<Path> paths = Files.walk(dir, 1)) {  // 参数1表示只查看当前目录
            return paths
                    .filter(Files::isRegularFile)  // 只处理普通文件
                    .map(p -> {
                        String fullName = p.getFileName().toString();
                        return new FileInfoDTO(
                                removeExtension(fullName),  // fileName不含后缀
                                fullName,                   // fullName包含后缀
                                p.toAbsolutePath().toString(),
                                p.toFile().length(),
                                getFileExtension(fullName)  // 扩展名（不带点）
                        );
                    })
                    .sorted(Comparator.comparing(FileInfoDTO::getFileName))  // 按文件名（不含后缀）排序
                    .collect(Collectors.toList());
        }
    }

    /**
     * 获取文件夹下指定扩展名的文件（fileName不含后缀），并按文件名（不含后缀）排序
     *
     * @param folderPath 文件夹路径
     * @param extension  文件扩展名（可带点或不带点），null表示获取所有文件，""表示无后缀文件
     * @return 排序后的文件列表
     * @throws IOException 如果读取文件出错
     */
    public static List<FileInfoDTO> getSortedFiles(String folderPath, String extension) throws IOException {
        if (extension == null) {
            return getAllSortedFiles(folderPath);
        }

        // 验证输入参数
        if (folderPath == null) {
            throw new IllegalArgumentException("文件夹路径不能为null");
        }

        Path dir = Paths.get(folderPath);
        if (!Files.isDirectory(dir)) {
            throw new IllegalArgumentException("提供的路径不是一个有效的文件夹: " + folderPath);
        }

        // 处理扩展名格式
        if (!extension.startsWith(".") && !extension.isEmpty()) {
            extension = "." + extension;
        }
        final String finalExtension = extension.toLowerCase();

        try (Stream<Path> paths = Files.walk(dir, 1)) {  // 参数1表示只查看当前目录
            String finalExtension1 = extension;
            return paths
                    .filter(Files::isRegularFile)  // 只处理普通文件
                    .filter(p -> {
                        String filename = p.getFileName().toString().toLowerCase();
                        // 匹配条件：1) 扩展名空字符串（无后缀文件） 2) 匹配指定扩展名
                        return finalExtension1.isEmpty()
                                ? !filename.contains(".")  // 无后缀文件
                                : filename.endsWith(finalExtension);  // 有指定后缀
                    })
                    .map(p -> {
                        String fullName = p.getFileName().toString();
                        return new FileInfoDTO(
                                removeExtension(fullName),  // fileName不含后缀
                                fullName,                 // fullName包含后缀
                                p.toAbsolutePath().toString(),
                                p.toFile().length(),
                                getFileExtension(fullName)  // 扩展名（不带点）
                        );
                    })
                    .sorted(Comparator.comparing(FileInfoDTO::getFileName))  // 按文件名（不含后缀）排序
                    .collect(Collectors.toList());
        }
    }

    // 辅助方法：移除文件扩展名
    private static String removeExtension(String filename) {
        int dotIndex = filename.lastIndexOf('.');
        return (dotIndex == -1) ? filename : filename.substring(0, dotIndex);
    }

    // 辅助方法：获取文件扩展名（不带点）
    private static String getFileExtension(String filename) {
        int dotIndex = filename.lastIndexOf('.');
        return (dotIndex == -1) ? "" : filename.substring(dotIndex + 1);
    }

//    // 测试方法
//    public static void main(String[] args) {
//        try {
//            String folderPath = "D:\\ips\\algrothm\\preprocess\\test\\test1";
//
//            // 1. 获取所有文件（fileName不含后缀）
//            System.out.println("所有文件（fileName不含后缀）:");
//            List<FileInfoDTO> allFiles = getAllSortedFiles(folderPath);
//            allFiles.forEach(file -> System.out.printf("%-30s %-15s %10d bytes %s%n",
//                    file.getFileName(),
//                    "(" + file.getFullName() + ")",
//                    file.getFileSize(),
//                    file.getExtension().isEmpty() ? "[无后缀]" : "扩展名: " + file.getExtension()));
//
//            // 2. 获取PDF文件（fileName不含后缀）
//            System.out.println("\nPDF文件（fileName不含后缀）:");
//            List<FileInfoDTO> pdfFiles = getSortedFiles(folderPath, "dat");
//            pdfFiles.forEach(file -> System.out.printf("%-30s %-15s %10d bytes %s%n",
//                    file.getFileName(),
//                    "(" + file.getFullName() + ")",
//                    file.getFileSize(),
//                    file.getExtension().isEmpty() ? "[无后缀]" : "扩展名: " + file.getExtension()));
//            // 3. 获取无后缀文件
//            System.out.println("\n无后缀文件:");
//            List<FileInfoDTO> noExtensionFiles = getSortedFiles(folderPath, "");
//            noExtensionFiles.forEach(file -> System.out.printf("%-30s %10d bytes%n",
//                    file.getFileName(), file.getFileSize()));
//
//        } catch (IOException e) {
//            System.err.println("处理文件时出错: " + e.getMessage());
//        }
//    }

    /**
     * 获取指定路径下所有子文件夹的结构信息
     * @param rootPath 根目录路径
     * @param targetExtensions 目标文件后缀列表（不带点）
     * @return 子文件夹的ParentFolderInfoDTO列表
     */
    public static List<ParentFolderInfoDTO> getFolderStructureWithFilter(String rootPath, List<String> targetExtensions) {
        File rootDir = new File(rootPath);
        if (!rootDir.exists() || !rootDir.isDirectory()) {
            throw new IllegalArgumentException("Invalid root directory path: " + rootPath);
        }

        List<ParentFolderInfoDTO> result = new ArrayList<>();

        // 获取所有子文件夹（第一级）
        File[] subFolders = rootDir.listFiles(File::isDirectory);
        if (subFolders != null) {
            Arrays.stream(subFolders).forEach(subFolder -> {
                // 每个子文件夹作为一个ParentFolderInfoDTO
                ParentFolderInfoDTO parentFolder = new ParentFolderInfoDTO(subFolder.getName());

                // 获取子文件夹下的孙子文件夹（第二级）
                File[] grandchildFolders = subFolder.listFiles(File::isDirectory);
                if (grandchildFolders != null) {
                    Arrays.stream(grandchildFolders).forEach(grandchildFolder -> {
                        // 每个孙子文件夹作为一个FolderInfoDTO
                        FolderInfoDTO grandchildFolderInfo = new FolderInfoDTO(grandchildFolder.getName());

                        // 获取孙子文件夹中符合后缀条件的文件
                        List<FileInfoDTO> filteredFiles = getFilesWithExtensions(grandchildFolder, targetExtensions);
                        grandchildFolderInfo.getFiles().addAll(filteredFiles);

                        // 将孙子文件夹添加到子文件夹的files列表中
                        parentFolder.getFiles().add(grandchildFolderInfo);
                    });
                }

                result.add(parentFolder);
            });
        }

        return result;
    }

    /**
     * 获取指定文件夹中符合后缀条件的文件
     * @param folder 文件夹
     * @param extensions 目标文件后缀列表（不带点）
     * @return 符合条件的文件列表
     */
    private static List<FileInfoDTO> getFilesWithExtensions(File folder, List<String> extensions) {
        File[] files = folder.listFiles(file -> {
            if (file.isFile()) {
                String fileName = file.getName();
                int dotIndex = fileName.lastIndexOf('.');
                if (dotIndex > 0) {
                    String extension = fileName.substring(dotIndex + 1).toLowerCase();
                    return extensions.isEmpty() || extensions.contains(extension);
                }
            }
            return false;
        });

        if (files == null) {
            return Collections.emptyList();
        }

        return Arrays.stream(files)
                .map(file -> {
                    String fileName = file.getName();
                    int dotIndex = fileName.lastIndexOf('.');
                    String nameWithoutExtension = dotIndex > 0 ? fileName.substring(0, dotIndex) : fileName;
                    String extension = dotIndex > 0 ? fileName.substring(dotIndex + 1) : "";

                    try {
                        long fileSize = Files.size(file.toPath());
                        return new FileInfoDTO(
                                nameWithoutExtension,
                                fileName,
                                file.getAbsolutePath(),
                                fileSize,
                                extension
                        );
                    } catch (IOException e) {
                        throw new RuntimeException("Failed to get file size for: " + file.getAbsolutePath(), e);
                    }
                })
                .collect(Collectors.toList());
    }

//    public static void main(String[] args) {
//        List<ParentFolderInfoDTO> jpg = getFolderStructureWithFilter("D:\\验收材料\\西工大\\输入输出样例0524\\3特征提取\\频域特征\\输入\\数据集", Arrays.asList("jpg"));
//        System.out.println(jpg);
//    }
}